//===========================================================
// SAVEIT - Saves the state of the desktop
//
// Copyright (C) 1995 Ziff-Davis Publishing Company
// First published in PC Magazine by Douglas Boling
//
// Revision History:
//
// 1.0   Initial Release
//
//===========================================================

//-----------------------------------------------------------
// Include files
//-----------------------------------------------------------
#include "windows.h"
#include "commdlg.h"
#include "shellapi.h"

#include "stdlib.h"
#include "string.h"
#include "direct.h"

#include "saveit.h"

//-----------------------------------------------------------
// Global data
//-----------------------------------------------------------
// Message dispatch table for MainWindowProc
struct decodeUINT MainMessages[] = {
	WM_PAINT, DoPaintMain,
	WM_CTLCOLOR, DoCtlColorMain,
	MYMSG_LAUNCHIT, DoLaunchItMain,
	WM_ENDSESSION, DoEndSessionMain,
	WM_QUERYENDSESSION, DoQueryEndSessionMain,
	WM_COMMAND, DoCommandMain,
	WM_CLOSE, DoCloseMain,
	WM_DESTROY, DoDestroyMain,
};
// Command Message dispatch for MainWindowProc
struct decodeCMD MainMenuItems[] = {
	IDD_EXIT, DoMainCtlExit,
	IDOK, DoMainCtlExit,
};

HANDLE	hInst;
HWND		hMain;
HLOCAL	hLocal = 0;
INT sEntryCnt = 0;
PPROGENTRY paArray;

char	szTitleText[] = "Save It";           // Window title text
char	szAppName[] = "SaveIt";              // Application name
char	szIconName[] = "SaveItIcon";         // Icon name
char	szProfileName[128];                  // INI file name
//============================================================
// WinMain -- entry point for this application from Windows.
//============================================================
INT APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, 
                     LPSTR lpCmdLine, INT nCmdShow) {
	MSG	msg;
	INT	rc;

	hInst = hInstance;

	// Don't allow more than one instance.
	if (hPrevInstance) 
	   return 1;
		
	// Init first instance of program.
	rc = InitApp(hInstance);
	if (rc) return rc;

	// Initialize this instance
	if((rc = InitInstance(hInstance, lpCmdLine, nCmdShow)) != 0)
		return rc;
	
	// Application message loop
	while (GetMessage (&msg, NULL, 0, 0)) {
		if (!IsDialogMessage (hMain, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}	
	}

	// Instance cleanup
	return TermInstance(hInstance, msg.wParam);
}
//-----------------------------------------------------------
// InitApp - Global initialization code for this application.
//-----------------------------------------------------------
INT InitApp(HANDLE hInstance) {
	WNDCLASS 	wc;
	//
	// Register App Main Window class
	//
	wc.style = 0;                             // Window style
	wc.lpfnWndProc = MainWndProc;             // Callback function
	wc.cbClsExtra = 0;                        // Extra class data
	wc.cbWndExtra = DLGWINDOWEXTRA;           // Extra window data
	wc.hInstance = hInstance;                 // Owner handle
	wc.hIcon = LoadIcon(hInst, szIconName);   // Application icon
	wc.hCursor = LoadCursor(NULL, IDC_ARROW); // Default cursor
	wc.hbrBackground = GetStockObject(LTGRAY_BRUSH); // Background color 
	wc.lpszMenuName =  NULL;                  // Menu name
	wc.lpszClassName = szAppName;             // Window class name

	if (RegisterClass(&wc) == 0)
		return 1;
	return 0;
}
//-----------------------------------------------------------
// InitInstance - Instance initialization code for this app.
//-----------------------------------------------------------
INT InitInstance(HANDLE hInstance, LPSTR lpCmdLine, INT nCmdShow) {
	INT	x, y;
	char szStatus[40];

	// Find program directory for local copy of INI file.
 	x = GetModuleFileName (hInstance, szProfileName, sizeof (szProfileName));
	while (szProfileName[x--] != '.');
	lstrcpy (&szProfileName[x+2], "INI");
	
	// Allocate memory for program array
	hLocal = LocalAlloc (GHND, sizeof (PROGENTRY) * MAXENTRIES);
	if (hLocal == 0)
		return 0x11;
	paArray = (PPROGENTRY) LocalLock (hLocal);
	
	//
	// Read information from the INI file
	//
	x = GetPrivateProfileInt (szAppName, PRO_XPOS, 100,
	                          szProfileName);
	y = GetPrivateProfileInt (szAppName, PRO_YPOS, 100,
	                          szProfileName);
	// Create main window
	hMain = CreateDialog (hInstance, szAppName, 0, NULL);
	if(!hMain) return 0x10;

	GetPrivateProfileString (szAppName, PRO_STATUS, "New", szStatus,
	                         sizeof (szStatus), szProfileName);
	WritePrivateProfileString (szAppName, PRO_STATUS, "Running",
			                     szProfileName);
	
	SetWindowPos (hMain, NULL, x, y, 0, 0, SWP_NOSIZE);

	// If installed, uninstall auto start on windows startup
	if (ChkInstalled())
		InstallProg (FALSE);
	
	if (lstrcmpi (szStatus, "Restart") == 0) {
		ShowWindow(hMain, SW_MINIMIZE | SW_SHOW);
		PostMessage (hMain, MYMSG_LAUNCHIT, 0, 0);
	} else
		ShowWindow(hMain, nCmdShow | SW_SHOW);

	UpdateWindow(hMain);              // force WM_PAINT message
	return 0;                         // return success flag
}
//------------------------------------------------------------
// TermInstance - Instance termination code for this app.
//------------------------------------------------------------
INT TermInstance(HANDLE hinstance, int sDefRC) {
	return sDefRC;
}
//============================================================
// Message handling procedures for MainWindow
//============================================================
//------------------------------------------------------------
// MainWndProc - Callback function for application window
//------------------------------------------------------------
LONG CALLBACK MainWndProc(HWND hWnd, UINT wMsg, UINT wParam, 
                          LONG lParam) {
	INT i;
	//
	// Search message list to see if we need to handle this
	// message.  If in list, call procedure.
	//
	for(i = 0; i < dim(MainMessages); i++) {
		if(wMsg == MainMessages[i].Code)
			return (*MainMessages[i].Fxn)(hWnd, wMsg, wParam, lParam);
	}
	return DefDlgProc(hWnd, wMsg, wParam, lParam);
}
//------------------------------------------------------------
// DoCtlColorMain - process WM_CTLCOLOR message for frame window.
//------------------------------------------------------------ 
LONG DoCtlColorMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {

	SetBkColor ((HDC) wParam, RGB (192, 192, 192));
	return GetStockObject (LTGRAY_BRUSH);
}
//------------------------------------------------------------
// DoPaintMain - process WM_PAINT message for frame window.
//------------------------------------------------------------ 
LONG DoPaintMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
	HDC hdc;
	HPEN hDPen, hLPen, hOldPen;
	PAINTSTRUCT ps;
	RECT rectAbout;
	//
	// Draw the rectangle surrounding the dialog box text.
	//
	hdc = BeginPaint (hWnd, &ps);
	//Get text rectangle defined in RC file
	GetClientRect (GetDlgItem (hWnd, IDD_FRAME), &rectAbout);
	ClientToScreen (GetDlgItem (hWnd, IDD_FRAME), (LPPOINT)&rectAbout);
	ScreenToClient (hWnd, (LPPOINT)&rectAbout);
	rectAbout.left -= 2;
	rectAbout.top -= 2;
	rectAbout.right += rectAbout.left + 4;
	rectAbout.bottom += rectAbout.top + 4;
	hDPen = CreatePen (PS_SOLID, 1, GetSysColor (COLOR_BTNSHADOW));
	hLPen = CreatePen (PS_SOLID, 1, GetSysColor (COLOR_BTNHIGHLIGHT));

	//Start at bottom left, draw dark pen up and over.
	hOldPen = SelectObject (hdc, hDPen);
	MoveTo (hdc, rectAbout.left, rectAbout.bottom);
	LineTo (hdc, rectAbout.left, rectAbout.top);
	LineTo (hdc, rectAbout.right+1, rectAbout.top);
	//Start at bottom left, draw light pen over and up.
	SelectObject (hdc, hLPen);
	MoveTo (hdc, rectAbout.left+1, rectAbout.bottom);
	LineTo (hdc, rectAbout.right, rectAbout.bottom);
	LineTo (hdc, rectAbout.right, rectAbout.top);

	SelectObject (hdc, hOldPen);
	DeleteObject (hDPen);
	DeleteObject (hLPen);
	EndPaint (hWnd, &ps);
	return DefDlgProc(hWnd, wMsg, wParam, lParam);
}
//------------------------------------------------------------
// DoLaunchItMain - process MYMSG_LAUNCHIT message for frame window.
//------------------------------------------------------------ 
LONG DoLaunchItMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
	INT i, sEntryCnt;
	HINSTANCE hExec;
	char szNum[33], szCmd[256];

	PROGENTRY Entry;
	
	sEntryCnt = GetPrivateProfileInt (szAppName, PRO_CNT, 0, 
	                                  szProfileName);
	if (sEntryCnt) {
		for (i = 0; i < sEntryCnt; i++) {
			// Read in data for program
			itoa (i, szNum, 10);
			Entry.fFlags = GetPrivateProfileInt (PRO_FLAGS, szNum, 
			                                      0, szProfileName);
			GetPrivateProfileString (PRO_CMD, szNum, "", Entry.szName,
			                         sizeof (Entry.szName), 
			                         szProfileName);
			GetPrivateProfileString (PRO_PARMS, szNum, "", Entry.szParms,
			                         sizeof (Entry.szParms), 
			                         szProfileName);
			GetPrivateProfileString (PRO_DIR, szNum, "", Entry.szDir,
			                         sizeof (Entry.szDir), 
			                         szProfileName);
			// Change to the working drive and dir
			if (lstrlen (Entry.szDir) != 0) {
				if (Entry.szDir[1] == ':') {
					//UCase drive letter & convert to num.
					_chdrive ((Entry.szDir[0] & 0xdf) - 0x40);
				}
				chdir (Entry.szDir);
 			}
			// Launch the program.
			lstrcpy (szCmd, Entry.szName);
			lstrcat (szCmd, " ");
			lstrcat (szCmd, Entry.szParms);

			hExec = WinExec (szCmd, Entry.fFlags);
			if (hExec < 32)  
				PrintError (hWnd, ERR_LAUNCH + hExec);
		}	
	}
	return 0;
}
//------------------------------------------------------------
// DoEndSessionMain - process WM_ENDSESSION message 
// for frame window.
//------------------------------------------------------------ 
LONG DoEndSessionMain (HWND hWnd, UINT wMsg, UINT wParam, 
                       LONG lParam) {
	
	if (wParam)	{
		if (!ChkInstalled())
			InstallProg (TRUE);
			
		WritePrivateProfileString (szAppName, PRO_STATUS, "Restart",
				                     szProfileName);
	}
	return DefDlgProc(hWnd, wMsg, wParam, lParam);
}
//------------------------------------------------------------
// DoQueryEndSessionMain - process WM_QUERYENDSESSION message 
// for frame window.
//------------------------------------------------------------ 
LONG DoQueryEndSessionMain (HWND hWnd, UINT wMsg, UINT wParam, 
                            LONG lParam) {
	HGLOBAL hStartup;

	// Get list of programs automatically launched on startup
	hStartup = GetStartupInfo ();

	// Get list of currently running programs.
	GetProgList ();

	// Eliminate programs running that are in startup list
	ParseProgList (hStartup);
	GlobalFree (hStartup);

	// Save info to ini file.
	SavetoINI ();

	return DefDlgProc(hWnd, wMsg, wParam, lParam);
}
//------------------------------------------------------------
// DoCommandMain - process WM_COMMAND message for frame window 
// by decoding the menubar item with the menuitems[] array, 
// then running the corresponding function to process the command.
//------------------------------------------------------------ 
LONG DoCommandMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
	INT	i;
	UINT	idItem, wNotifyCode;
	HWND	hwndCtl;

	idItem = (UINT) wParam;                      // Parse Parameters
	hwndCtl = (HWND) LOWORD(lParam);
	wNotifyCode = (UINT) HIWORD(lParam);
	//
	// Call routine to handle control message
	//
	for(i = 0; i < dim(MainMenuItems); i++) {
		if(idItem == MainMenuItems[i].Code)
			return (*MainMenuItems[i].Fxn)(hWnd, idItem, hwndCtl, 
			                               wNotifyCode);
	}
	return DefDlgProc(hWnd, wMsg, wParam, lParam);
}
//------------------------------------------------------------
// DoCloseMain - process WM_CLOSE message for frame window.
//------------------------------------------------------------ 
LONG DoCloseMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
	INT sRC;

	sRC = MessageBox (hWnd, 
	             "SaveIt can only restore your desktop\nif it is running"\
	             " when you exit Windows.\n\n"\
	             "Are you sure you want to exit the program?", 
	                  szTitleText, MB_YESNO | MB_ICONINFORMATION);
	if (sRC == IDYES)
		DestroyWindow (hMain);
	return 0;
}
//------------------------------------------------------------
// DoDestroyMain - process WM_DESTROY message for frame window.
//------------------------------------------------------------ 
LONG DoDestroyMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
   RECT	rect;

	//Save Window position
	if	(!IsIconic (hWnd) && !IsZoomed (hWnd)) {
		GetWindowRect (hWnd, &rect);
		MyWritePrivateProfileInt (szAppName, PRO_XPOS, rect.left,
		                          szProfileName);
		MyWritePrivateProfileInt (szAppName, PRO_YPOS, rect.top,
		                          szProfileName);
	}
	PostQuitMessage (0);
	return DefDlgProc(hWnd, wMsg, wParam, lParam);
}
//============================================================
// Control handling procedures for MainWindow
//============================================================
//------------------------------------------------------------
// DoMainCtlExit - Process Exit button
//------------------------------------------------------------ 
LONG DoMainCtlExit (HWND hWnd, UINT idItem, HWND hwndCtl, 
                     UINT wNotifyCode) {
	
	if (wNotifyCode != BN_CLICKED)
		return 0;

	SendMessage (hWnd, WM_CLOSE, 0, 0);
	return 0;
}
//------------------------------------------------------------
// GetProgList - Loads array with list of running programs.
//------------------------------------------------------------ 
INT GetProgList (void) {
	PPROGENTRY paPtr;
	HWND hwndNext;
	HTASK hTask;
	LPSTR lpszPSPCmdLine;
	LPSTR lpszTemp;
	INT i;
	BOOL fFound;

	paPtr = paArray;
	hwndNext = GetWindow (GetDesktopWindow(), GW_CHILD);
	sEntryCnt = 0;
	while (hwndNext) {
		if (IsWindowVisible (hwndNext) &&
		    !(GetWindow (hwndNext, GW_OWNER))) {
		
			//If WinOldApp, look in task DB for name of DOS program
			hTask = GetWindowTask (hwndNext);
			fFound = FALSE;
			for (i = 0; i < sEntryCnt; i++) 
				if (hTask == (paArray+i)->hTask) {
					fFound = TRUE;
					break;
				}	
			if (!fFound) {		
				paPtr->hTask = hTask;
			
				//Make pointer to PSP command line
				lpszPSPCmdLine = MAKELP (hTask, 0x180);
			
				if (IsWinOldApTask (hTask)) {
					//Skip past parameters to name of program			
					lpszTemp = lpszPSPCmdLine + lstrlen (lpszPSPCmdLine)+1;
					lstrcpy (paPtr->szName, lpszTemp);
				} else 
					GetModuleFileName (GetWindowWord (hwndNext, GWW_HINSTANCE),
					                   paPtr->szName, sizeof (paPtr->szName));

				// Copy command line parameters if they exist
				if (*lpszPSPCmdLine)
					lstrcpyn (paPtr->szParms, lpszPSPCmdLine+1, *lpszPSPCmdLine+1);
				else	
					*paPtr->szParms = '\0';
				
				// Copy current directory of task
				lpszPSPCmdLine = MAKELP (hTask, 0x66);
				lpszTemp = paPtr->szDir;
				*lpszTemp++ = *lpszPSPCmdLine - 0x80 + 'A';
				*lpszTemp++ = ':';
				lstrcpy (lpszTemp, lpszPSPCmdLine+1);

				if (IsIconic (hwndNext)) 
					paPtr->fFlags = SW_SHOWMINIMIZED;
				else if (IsZoomed (hwndNext))
					paPtr->fFlags = SW_SHOWMAXIMIZED;
				else 
					paPtr->fFlags = SW_NORMAL;

				paPtr++;
				sEntryCnt++;
			}
		}	
		hwndNext = GetWindow (hwndNext, GW_HWNDNEXT);
	}	
	return 0;
}
//------------------------------------------------------------
// ParseProgList - Removes programs in the list that are 
// listed in the startup list.
//------------------------------------------------------------ 
INT ParseProgList (HGLOBAL hStartup) {
	PPROGENTRY paPtr;
	LPSTR lpStartup, lpTemp;
	INT i;

	if (hStartup == 0)
		return 0;

	lpStartup = GlobalLock (hStartup);

	if (lpStartup == 0)
		return 0;

	paPtr = paArray;
	for (i = 0; i < sEntryCnt; i++) {
		lpTemp = lpStartup;
		
		while (*lpTemp) {
			if (lstrcmpi (paPtr->szName, lpTemp) == 0) {
				paPtr->hTask = 0;
				*lpTemp = '/';
				break;
			}
			lpTemp += lstrlen (lpTemp) + 1;
		}
		paPtr++;
	}
	GlobalUnlock (hStartup);
	return 0;
}
//------------------------------------------------------------
// SavetoINI - Writes the data to the ini file.
//------------------------------------------------------------ 
void SavetoINI (void) {
	INT i, sCnt = 0;
	PPROGENTRY paPtr;
	char	szNum[8];

	// Clear out lists	
	WritePrivateProfileString (PRO_FLAGS, NULL, NULL, szProfileName);
	WritePrivateProfileString (PRO_CMD, NULL, NULL, szProfileName);
	WritePrivateProfileString (PRO_PARMS, NULL, NULL, szProfileName);
	WritePrivateProfileString (PRO_DIR, NULL, NULL, szProfileName);

	// Rewrite list to INI file
	paPtr = paArray;
	for (i = 0; i < sEntryCnt; i++) {
		if (paPtr->hTask) {
			itoa (sCnt, szNum, 10);
			MyWritePrivateProfileInt (PRO_FLAGS, szNum, paPtr->fFlags, szProfileName);
			WritePrivateProfileString (PRO_CMD, szNum, paPtr->szName, szProfileName);
			WritePrivateProfileString (PRO_PARMS, szNum, paPtr->szParms, szProfileName);
			WritePrivateProfileString (PRO_DIR, szNum, paPtr->szDir, szProfileName);
			sCnt++;
		}	
		paPtr++;
	}
	MyWritePrivateProfileInt (szAppName, PRO_CNT, sCnt, szProfileName);
	return;
}	
//------------------------------------------------------------
// ParseINIString - Parses file names from an ini string
//------------------------------------------------------------
LPSTR ParseINIString (LPSTR lpIn, LPSTR lpOut) {
	LPSTR lpEnd;
	char ch;

	while (*lpIn) {
		while ((*lpIn) && (*lpIn <= ' '))
			lpIn++;
		
		lpEnd = lpIn;
		while ((*lpEnd > ' ') && (*lpEnd != '/'))
			lpEnd++;
		ch = *lpEnd;
		*lpEnd = '\0';
		if (FindExecutable (lpIn, "", lpOut) < 32)
			lstrcpy (lpOut, lpIn);
		lpOut += lstrlen (lpOut) + 1;
		*lpEnd = ch;
		lpIn = lpEnd;
	}	
	return lpOut;
}	
//------------------------------------------------------------
// GetEndofName - returns a pointer to the end of a filename
//------------------------------------------------------------
LPSTR GetEndofName (LPSTR lpName) {
	char ch;

	while (*lpName) {
		ch = *lpName;
		if ((strchr (":\\.!#$%^&()-_{}~", ch) == 0) &&
		    !IsCharAlphaNumeric (*lpName))
			break;
		lpName++;
	}	
	return lpName;
}	
//------------------------------------------------------------
// GetGroupData - Reads information from a Group file 
//------------------------------------------------------------ 
LPSTR GetStartupList (HFILE hFile, LPSTR lpOut) {
	HGLOBAL hGlobal, hFileMem;
	LPSTR lpData, lpStart, lpTemp, lpFile;
	LPSTR lpPtr, lpDir, lpEnd;
	char szSwap[128];
	LONG lFileSize;
	INT i, sNum, sCnt = 0;
	LPITEMDATA lpItem;

	hGlobal = GlobalAlloc (GHND, 0x10000);
	if (!hGlobal)
		return lpOut;
	
	hFileMem = GlobalAlloc (GHND, 0x10000);
	if (!hFileMem) {
		GlobalFree (hGlobal);
		return lpOut;
	}	
	lpFile = GlobalLock (hFileMem);
	lpStart = GlobalLock (hGlobal);
	lpPtr = lpStart;


	// Compute size of group file by seeking to end of file
	lFileSize = _llseek (hFile, 0, 2);		
	_llseek (hFile, 0, 0);

	if (_hread (hFile, lpFile, lFileSize) == lFileSize) {

		// Save number items in sub menu				
		sNum = ((LPGROUPHEADER)lpFile)->cItems;
		if (sNum > 120)
			sNum = 120;

		// Scan group item structures to get names of each items.
		for (i = 0; i < sNum; i++) {
			if (((LPGROUPHEADER)lpFile)->rgiItems[i]) {
				lpItem = (LPITEMDATA)(lpFile + ((LPGROUPHEADER)lpFile)->rgiItems[i]);
					
				// Copy command, the path specified is actually the working dir
				lpTemp = lpFile + lpItem->pCommand;
				// Assume a path (working dir) exists
				lpDir = lpPtr + 128;
				lstrcpy (lpDir, lpTemp);
				lpEnd = GetEndofName (lpDir);
				while ((lpEnd > lpDir) && (*lpEnd != '\\'))
					lpEnd--;
				// if path specified, its the working dir, so move cmd over	
				if (lpEnd > lpDir) {
					// Copy old path to dir field
					lstrcpy (lpPtr, lpEnd+1);
				} else {
					// no working dir, so move cmd and zero wdir
					lstrcpy (lpPtr, lpTemp);
					*lpDir = '\0';
				}	
			} else
				*lpPtr = '\0';
				
			lpPtr += 256;
		}
		// Get ptr to tag structures
		lpPtr = lpFile + ((LPGROUPHEADER)lpFile)->cbGroup;

		if (((LPTAGDATA)lpPtr)->wID == 0x8000) {

			while (((LPTAGDATA)lpPtr)->wID != 0xFFFF) {
				// Get ptr to item tag addressses
				lpData = lpStart + (((LPTAGDATA)lpPtr)->wItem * 256);
				if (*lpData) {
					// Path tag
					if (((LPTAGDATA)lpPtr)->wID == 0x8101) {

						// Get ptr to path
						lpTemp = ((LPTAGDATA)lpPtr)->rgb;
						// Copy wdir (actually path) to temp string
						lstrcpy (szSwap, lpTemp);
						// Append cmd
						lstrcat (szSwap, lpData);
						// Move back to cmd string
						lstrcpy (lpData, szSwap);
					}
				}
			 	lpPtr += ((LPTAGDATA)lpPtr)->cb;
			}	
		}
	}		
	_lclose (hFile);

	// Now copy strings in array to list of strings. 
	lpPtr = lpStart;
	for (i = 0; i < sNum; i++) {
		if (*lpPtr) {
			// Use FindExecutable to parse/find filename
			FindExecutable (lpPtr, lpPtr+128, lpOut);
			lpOut += lstrlen (lpOut) + 1;
		}
		lpPtr += 256;
	}
	*lpOut = '\0';		
	
	GlobalUnlock (hFileMem);
	GlobalFree (hFileMem);
	
	GlobalUnlock (hGlobal);
	GlobalFree (hGlobal);
	
	return lpOut;
}	
//------------------------------------------------------------
// GetStartupInfo - Reads the Startup Group file 
//------------------------------------------------------------ 
HGLOBAL GetStartupInfo (void) {
	HFILE hFile;
	OFSTRUCT of;
	char szStartTitle[64], szTemp[64];
	char szStartupPath[128];
	BOOL fFound = FALSE;
	GROUPHEADER gh;
	HGLOBAL hGlobal;
	LPSTR lpNames;
	INT i;

	// See if startup group explicitly specified.
	GetPrivateProfileString ("Settings", "Startup", "StartUp", szStartTitle,
	                         sizeof (szStartTitle), "ProgMan.INI");

	// Find path to startup group by looking in ProgMan.ini
	for (i = 1; i < 50; i++) {
		wsprintf (szTemp, "Group%d",i);
		GetPrivateProfileString ("Groups", szTemp, "", szStartupPath,
		                         sizeof (szStartupPath), "ProgMan.INI");
		if (szStartupPath[0] == '\0')
			break;										 

		// Open the group file		
		hFile = OpenFile (szStartupPath, &of, OF_READ | OF_SHARE_DENY_NONE);
		if (hFile != HFILE_ERROR) {

			if (_hread (hFile, &gh, sizeof (GROUPHEADER)) == sizeof (GROUPHEADER)) {
	
				_llseek (hFile, gh.pName, 0);
				_hread (hFile, szTemp, sizeof (szTemp));

				if (lstrcmpi (szTemp, szStartTitle) == 0) {
					fFound = TRUE;
					break;
				}
			}	
			_lclose (hFile);
		}
	}		
	// Alloc mem for file name buffer
	hGlobal = GlobalAlloc (GHND, 0x10000);
	if (!hGlobal) {
		if (fFound)
			_lclose (hFile);
		return 0;
	}	
	lpNames = GlobalLock (hGlobal);

	// Get program names from groupfile
	if (fFound)
		lpNames = GetStartupList (hFile, lpNames);

	// Now parse the RUN line in WIN.INI
	GetProfileString ("Windows","run", "", szStartupPath, 
	                  sizeof (szStartupPath));
	lpNames = ParseINIString (szStartupPath, lpNames);
	
	// Parse the LOAD line in WIN.INI
	GetProfileString ("Windows","load", "", szStartupPath, 
	                  sizeof (szStartupPath));
	lpNames = ParseINIString (szStartupPath, lpNames);

	// Add the name of the shell from SYSTEM.INI
	GetPrivateProfileString ("boot","shell", "", szStartupPath, 
	                         sizeof (szStartupPath), "SYSTEM.INI");
	lpNames = ParseINIString (szStartupPath, lpNames);

	// Add our name to the list
 	GetModuleFileName (hInst, lpNames, 128);
	lpNames += lstrlen (lpNames) + 1;
	*lpNames = '\0';

	GlobalUnlock (hGlobal);
	return hGlobal;
}	
//------------------------------------------------------------
// ChkInstalled - See if already installed
//------------------------------------------------------------ 
INT ChkInstalled (void) {
	char szName[128], szStr[256], *pszOut, szOut[256];
	LPSTR lpszOut;

	GetModuleFileName (hInst, szName, sizeof (szName));

	// Parse the LOAD line in WIN.INI
	GetProfileString ("Windows","load", "", szStr, sizeof (szStr));
	lpszOut = ParseINIString (szStr, szOut);
	*++lpszOut = '\0';

	// Scan list of progs in LOAD string to see if we are there.
	pszOut = szOut;
	while (*pszOut) {
		if (lstrcmp (szName, pszOut) == 0)
			return 1;
		pszOut += lstrlen (pszOut) + 1;
	}
	return 0;
}

//------------------------------------------------------------
// InstallProg - Install/Uninstall program
//------------------------------------------------------------ 
INT InstallProg (BOOL fInstall) {
	char szName[128], szStr[256], *pszStr, *pszEnd, ch; 

	// Get the program name		
	GetModuleFileName (hInst, szName, sizeof (szName));

	// Parse the LOAD line in WIN.INI
	GetProfileString ("Windows","load", "", szStr, sizeof (szStr));
	AnsiUpper (szStr);
	AnsiUpper (szName);
	
	if (fInstall) {
		// Install program by appending prog name to end of LOAD string
		lstrcat (szStr, " ");
		lstrcat (szStr, szName);
	} else {	
		// Remove program by parsing name in LOAD line and removing
		pszStr = szStr;
		while (*pszStr) {

			// Skip past any spaces in line
			while ((*pszStr) && (*pszStr <= ' '))
				pszStr++;
			pszEnd = pszStr;

			// If end of string, do nothing
			if (*pszStr) {
				// truncate at end of current name
				while ((*pszEnd) && (*pszEnd > ' '))
					pszEnd++;
				ch = *pszEnd;
				*pszEnd = '\0';

				if (lstrcmp (pszStr, szName) == 0) {
					// Name found, overwrite with remainder of string
					*pszEnd = ch;
					while (*pszStr++ = *pszEnd++)
						;					
					break;
				} else
					*pszEnd = ch;
			}	
			pszStr = pszEnd;
		}
	}	
	WriteProfileString ("Windows","load", szStr);
	return 0;
}
//============================================================  
// General Helper Routines 
//============================================================  
//------------------------------------------------------------
// PrintError - Displays a message box with an error code
//------------------------------------------------------------ 
void PrintError (HWND hWnd, INT rc) {
	char szErrStr[80];
	
	rc = abs (rc);
	if (LoadString (hInst, rc, szErrStr, sizeof (szErrStr)) == 0)
		wsprintf (szErrStr, "Error number: %d", rc);

	MessageBox (hWnd, szErrStr, szTitleText, MB_OK | MB_ICONHAND);
	return;
}
//------------------------------------------------------------
// MyWritePrivateProfileInt - Writes an integer to the profile
//------------------------------------------------------------
BOOL MyWritePrivateProfileInt (char *szSec, char *szEntry, 
                               int Num, char *szProfile) {
	char	szStr[33];
	                           
	wsprintf (szStr, "%d", Num);
	return WritePrivateProfileString (szSec, szEntry, szStr, szProfile);
}
